@@ -53,7 +53,7 @@ def i_upload_a_file_async(step, file):
     world.resource = resource
 
 @step(r'I wait until the source has been created less than (\d+) secs')
-def the_source_has_been_created(step, secs):
+def the_source_has_been_created_async(step, secs):
     start = datetime.utcnow()
     while world.resource['object']['status']['code'] == UPLOADING:
         time.sleep(3)
